"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createWarningValidators = exports.createErrorValidators = void 0;
var _schema = require("./schema");
var _validation = require("./validation");
var _utils = require("./utils");
var _validators = require("./validators.js");
function _typeof(o) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) { return typeof o; } : function (o) { return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o; }, _typeof(o); }
function _toConsumableArray(r) { return _arrayWithoutHoles(r) || _iterableToArray(r) || _unsupportedIterableToArray(r) || _nonIterableSpread(); }
function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }
function _unsupportedIterableToArray(r, a) { if (r) { if ("string" == typeof r) return _arrayLikeToArray(r, a); var t = {}.toString.call(r).slice(8, -1); return "Object" === t && r.constructor && (t = r.constructor.name), "Map" === t || "Set" === t ? Array.from(r) : "Arguments" === t || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(t) ? _arrayLikeToArray(r, a) : void 0; } }
function _iterableToArray(r) { if ("undefined" != typeof Symbol && null != r[Symbol.iterator] || null != r["@@iterator"]) return Array.from(r); }
function _arrayWithoutHoles(r) { if (Array.isArray(r)) return _arrayLikeToArray(r); }
function _arrayLikeToArray(r, a) { (null == a || a > r.length) && (a = r.length); for (var e = 0, n = Array(a); e < a; e++) n[e] = r[e]; return n; }
function _defineProperty(e, r, t) { return (r = _toPropertyKey(r)) in e ? Object.defineProperty(e, r, { value: t, enumerable: !0, configurable: !0, writable: !0 }) : e[r] = t, e; }
function _toPropertyKey(t) { var i = _toPrimitive(t, "string"); return "symbol" == _typeof(i) ? i : i + ""; }
function _toPrimitive(t, r) { if ("object" != _typeof(t) || !t) return t; var e = t[Symbol.toPrimitive]; if (void 0 !== e) { var i = e.call(t, r || "default"); if ("object" != _typeof(i)) return i; throw new TypeError("@@toPrimitive must return a primitive value."); } return ("string" === r ? String : Number)(t); }
/**
 * Extracts message data based on a validation
 * test in the schema.
 *
 * @param {string} kind Type of the field
 * @param {object} entry 
 *
 * @returns {object} Data for the message translation.
 */
var extractMsgData = function extractMsgData(kind, entry) {
  var data = entry[kind];
  switch (kind) {
    case 'type':
      if (data instanceof _schema.Schema) return _defineProperty({}, kind, data.getType());else return _defineProperty({}, kind, (0, _utils.schemaType)(data));
    case 'validation':
    case 'accept':
    case 'reject':
      return {};
    default:
      return _defineProperty({}, kind, data);
  }
};

/**
 * Creates all the needed tests for a configuration
 * block by comparing each attribute with the existing
 * test creators.
 *
 * @param {object} entry Object with the validation tests.
 *
 * @returns {array} All the tests created and ready.
 */
var createTesters = function createTesters(entry) {
  var entryKeys = Object.keys(entry);
  return entryKeys.reduce(function (tests, kind) {
    var creator = _validators.testFactory[kind];
    if (creator) {
      var test = creator(entry);
      var msgData = extractMsgData(kind, entry);
      return [].concat(_toConsumableArray(tests), [{
        kind: kind,
        test: test,
        msgData: msgData
      }]);
    } else {
      return tests;
    }
  }, []);
};

/**
 * Extracts an entry or list of entries to generate
 * a flat array with the needed tests already created.
 *
 * @param {object|array} Test block or list of tests blocks
 *    used to generate the validation tests.
 *
 * @returns {array} All the tests created and ready.
 */
var extract = function extract(entry) {
  if (entry) {
    var tests = (0, _utils.forceArray)(entry);
    return tests.reduce(function (tests, entry) {
      return [].concat(_toConsumableArray(tests), _toConsumableArray(createTesters(entry)));
    }, []);
  } else {
    return [];
  }
};

/**
 * Extracts just the error validations. That can
 * happen in the same schema block as the field
 * or in an object in the error attribute.
 *
 * @param {object} entry Schema entry.
 *
 * @returns {array} Tests created and ready.
 */
var extractErrorer = function extractErrorer(entry) {
  return [].concat(_toConsumableArray(extract(entry)), _toConsumableArray(extract(entry.error)));
};

/**
 * Extracts just the warning block
 */
var extractWarner = function extractWarner(entry) {
  return [].concat(_toConsumableArray(extract({
    type: entry.type
  })), _toConsumableArray(extract(entry.warning)));
};

/**
 * Bridge to extract the desired validations.
 *
 * @param {object} schema Schema
 * @param {extractorCallback} extractor Function used
 *    to select what to extract and extract it.
 *
 * @returns {array} Validated tests ready for use.
 */
var extractValidations = function extractValidations(schema, extractor) {
  var keys = Object.keys(schema);
  return keys.reduce(function (validators, key) {
    var entry = schema[key];
    validators[key] = extractor(entry);
    return validators;
  }, {});
};

/**
 * Callback for extracting validation tests.
 *
 * @callback extractorCallback
 * @param {object} entry Schema data
 */

/**
 * Creates the validation code for an schema using
 * a extractor.
 *
 * @param {object} schema Schema data
 * @param {extractorCallback} extractor Select and
 *    extract.
 */
var createValidators = function createValidators(_ref4) {
  var schema = _ref4.schema,
    extractor = _ref4.extractor,
    validationType = _ref4.validationType;
  var validationsByKey = extractValidations(schema, extractor);
  extractor.validationType = validationType;
  return function (model, props) {
    var validation = (0, _validation.validate)({
      model: model,
      validationsByKey: validationsByKey,
      validationType: validationType,
      props: props
    });
    return validation;
  };
};

/**
 * Extracts only the error validators.
 *
 * @param {object} schema Schema data.
 *
 * @returns {array} Validators
 */
var createErrorValidators = exports.createErrorValidators = function createErrorValidators(schema) {
  return createValidators({
    schema: schema,
    extractor: extractErrorer,
    validationType: 'error'
  });
};

/**
 * Extracts only the warning validators.
 *
 * @param {object} schema Schema data
 */
var createWarningValidators = exports.createWarningValidators = function createWarningValidators(schema) {
  return createValidators({
    schema: schema,
    extractor: extractWarner,
    validationType: 'warning'
  });
};