"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Schema = void 0;
Object.defineProperty(exports, "regexes", {
  enumerable: true,
  get: function get() {
    return _regexes.regexes;
  }
});
Object.defineProperty(exports, "translatable", {
  enumerable: true,
  get: function get() {
    return _validation.translatable;
  }
});
var _extraction = require("./extraction");
var _regexes = require("./regexes");
var _validation = require("./validation");
var _translation_utils = require("./translation_utils");
function _typeof(o) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) { return typeof o; } : function (o) { return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o; }, _typeof(o); }
function _classCallCheck(a, n) { if (!(a instanceof n)) throw new TypeError("Cannot call a class as a function"); }
function _defineProperties(e, r) { for (var t = 0; t < r.length; t++) { var o = r[t]; o.enumerable = o.enumerable || !1, o.configurable = !0, "value" in o && (o.writable = !0), Object.defineProperty(e, _toPropertyKey(o.key), o); } }
function _createClass(e, r, t) { return r && _defineProperties(e.prototype, r), t && _defineProperties(e, t), Object.defineProperty(e, "prototype", { writable: !1 }), e; }
function _toPropertyKey(t) { var i = _toPrimitive(t, "string"); return "symbol" == _typeof(i) ? i : i + ""; }
function _toPrimitive(t, r) { if ("object" != _typeof(t) || !t) return t; var e = t[Symbol.toPrimitive]; if (void 0 !== e) { var i = e.call(t, r || "default"); if ("object" != _typeof(i)) return i; throw new TypeError("@@toPrimitive must return a primitive value."); } return ("string" === r ? String : Number)(t); }
/**
 * Contains the schema specifications and automatically
 * creates validation code for errors and warnings, both
 * translated and untranslated.
 */
var Schema = exports.Schema = /*#__PURE__*/function () {
  /**
   * Creates a Schema from the specification.
   *
   * @param {string} typeName Name of the model being created.
   *    It can be chosen freely.
   * @param {object} schema Schema specification.
   */
  function Schema(typeName, schema) {
    _classCallCheck(this, Schema);
    this.schema = schema;
    this.typeName = typeName;
    this.validators = {
      error: (0, _extraction.createErrorValidators)(this.schema),
      warning: (0, _extraction.createWarningValidators)(this.schema)
    };
    this.translated = {
      error: (0, _translation_utils.createTranslatedErrorValidator)(this),
      warning: (0, _translation_utils.createTranslatedWarningValidator)(this)
    };
  }

  /**
   * Returns the schema specification.
   *
   * @returns {object} Schema specification.
   */
  return _createClass(Schema, [{
    key: "getSchema",
    value: function getSchema() {
      return this.schema;
    }

    /**
     * Returns schema specification for a field.
     *
     * @returns {object} Field specification.
     */
  }, {
    key: "getFieldSchema",
    value: function getFieldSchema(name) {
      return this.schema[name];
    }

    /**
     * Returns the schema name.
     *
     * @returns {string} Schema name (also called ``typeName``).
     */
  }, {
    key: "getType",
    value: function getType() {
      return this.typeName;
    }

    /**
     * Gets error validator that produces a translatable
     * tree with the found errors.
     *
     * @returns {validator} Drop in validator
     *
     * @see getTranslatedErrorValidator
     */
  }, {
    key: "getErrorValidator",
    value: function getErrorValidator() {
      return this.validators.error;
    }

    /**
     * Gets warning validator that produces a translatable
     * tree with the found warnings.
     *
     * @returns {validator} Drop in validator
     *
     * @see getTranslatedWarningValidator
     */
  }, {
    key: "getWarningValidator",
    value: function getWarningValidator() {
      return this.validators.warning;
    }

    /**
     * Gets the translated errors validator for direct
     * use with ReduxForm.
     *
     * @returns {validator} Drop in validator
     */
  }, {
    key: "getTranslatedErrorValidator",
    value: function getTranslatedErrorValidator() {
      return this.translated.error;
    }

    /**
     * Gets the translated warnings validator for direct
     * use with ReduxForm.
     *
     * @returns {validator} Drop in validator
     */
  }, {
    key: "getTranslatedWarningValidator",
    value: function getTranslatedWarningValidator() {
      return this.translated.warning;
    }

    /**
     * Validates using the specified validator type
     * (error or warning)
     *
     * @param {object} doc Document to validate
     * @param {string} validationType 'error' or 'warning'
     *
     * @returns {object} Validation tree
     */
  }, {
    key: "validateType",
    value: function validateType(doc, validationType, props) {
      var validator = this.validators[validationType];
      return validator(doc, props);
    }

    /**
     * Information returned by validate()
     *
     * @typedef {object} validationResult
     * @property {boolean} valid States if the document
     *    is valid (no errors).
     * @property {object} errors Translatable object
     *    with the errors.
     * @property {object} warnings Translatable object
     *    with the warnings.
     */

    /**
     * Validates a document and returns an information
     * object about the results.
     *
     * @param {object} doc Document to validate
     * @param {object} props Data passed to validator
     *    as third parameter
     *
     * @returns {validationResult}
     */
  }, {
    key: "validate",
    value: function validate(doc, props) {
      var errors = this.validateType(doc, 'error', props);
      var warnings = this.validateType(doc, 'warning', props);
      return {
        valid: Object.keys(errors).length == 0,
        errors: errors,
        warnings: warnings
      };
    }

    /**
     * Returns true if the document is valid following
     * the schema.
     *
     * @returns {boolean} Validity
     */
  }, {
    key: "isValid",
    value: function isValid(doc) {
      var errors = this.validateType(doc, 'error');
      return Object.keys(errors).length == 0;
    }
  }]);
}();