"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.tr = exports.setTranslator = exports.setLanguage = exports.addTranslations = void 0;
var _deepExtend = _interopRequireDefault(require("deep-extend"));
function _interopRequireDefault(e) { return e && e.__esModule ? e : { "default": e }; }
var translations = {};
var VAR_REGEX = /__(.*?)__/g;
var REF_REGEX = /@@(.*?)@@/g;
var findString = function findString(id) {
  var part = id.split('.');
  var lastIndex = part.length - 1;
  return part.reduce(function (node, cur, index) {
    var isLast = index == lastIndex;
    var isString = typeof node == 'string';
    if (isString) return node;else {
      if (node && node[cur]) {
        if (isLast && node[cur]._) return node[cur]._;else return node[cur];
      } else {
        return node && node._ || id;
      }
    }
  }, translations);
};
var regexReplace = function regexReplace(regex, str, callback) {
  var match;
  var result = str;
  var re = new RegExp(regex);
  while ((match = re.exec(str)) !== null) {
    var value = callback(match[1]);
    if (typeof value != 'undefined') result = result.replace(match[0], value);
  }
  return result;
};

/**
 * Translates a string given its id.
 *
 * @param {string} id Identifier in the form
 * 	key1.key2.key3
 * @param {object} vars Object with substitution variables. It will
 * 	substitute ocurrences when string contains this expression:
 * 	__variable__. For example the string "My name is __name__" with
 * 	vars = { name: 'David' } will return "My name is David".
 *
 * 	Keys will be searched by partitioning the 'path'.
 * 	It will get the latest found key if any. For example, given the
 * 	strings { "a": { "b": 'Hello' } } and looking for 'a.b.c' it will
 * 	return 'a.b' ("Hello").
 * @returns Translated string
 */
var _tr = exports.tr = function tr(id) {
  var vars = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  var found = findString(id);
  if (found) {
    // Find variables
    found = regexReplace(VAR_REGEX, found, function (match) {
      return vars[match];
    });

    // Find references
    found = regexReplace(REF_REGEX, found, function (match) {
      return _tr(match, vars);
    });
    return found;
  } else return id;
};

/**
 * Sets the locale.
 *
 * At the moment this does the same as addTranslations. The
 * reason is not to lose translations reference until a better
 * way is figured out.
 *
 * @param {locale} Translations object with the format
 * 	{ key: { _: 'Some string', inner: 'Some other string' } }
 * 	Then, we have the following paths
 * 	- key -> 'Some string'
 * 	- key.inner -> 'Some other string'
 */
var setLanguage = exports.setLanguage = function setLanguage(lang) {
  addTranslations(lang);
};

/**
 * Appends translations to current translation table
 *
 * @param {object} lang Translations merged into current.
 */
var addTranslations = exports.addTranslations = function addTranslations(lang) {
  translations = (0, _deepExtend["default"])(translations, lang);
};

/**
 * Sets the translation engine that responds to tr().
 *
 * @param {function} translate Function with signature
 * 	translate(id, params).
 */
var setTranslator = exports.setTranslator = function setTranslator(translate) {
  exports.tr = _tr = translate;
};