"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.validate = exports.translatable = void 0;
var _utils = require("./utils");
var _validators = require("./validators");
/**
 * Translatable intermediary object.
 *
 * @typedef {object} translatableObject
 *
 * @property {boolean} _translatable Always ``true``, used to
 *    identify translations in the results object.
 * @property {string} msg Id for the message in the translation
 *    table.
 * @property {object} msgData Data passed to the tr() function
 *    to provide variable substitution for the string.
 */

/**
 * Creates a translatable object.
 *
 * @param {string} msg Id for the message
 * @param {object} msgData Object for variable substitution
 *
 * @returns {translatableObject} Object to be translated elsewhere.
 */
var translatable = exports.translatable = function translatable(msg, msgData) {
  var result = !(0, _utils.objectIsEmpty)(msgData) ? {
    msg: msg,
    msgData: msgData
  } : {
    msg: msg
  };
  return Object.assign(result, {
    _translatable: true
  });
};
var validateEntry = function validateEntry(_ref) {
  var validationType = _ref.validationType,
    value = _ref.value,
    validations = _ref.validations,
    props = _ref.props;
  return validations.reduce(function (msg, validation) {
    if (msg) {
      return msg;
    } else {
      var validator = validation.test;
      var result = validator(value, validationType, props);
      var trimmed = (0, _utils.arrayTrim)(result);
      if (trimmed) {
        if (typeof trimmed == 'boolean') {
          var kind = validation.kind,
            msgData = validation.msgData;
          var msgId = "".concat(validationType, ".").concat(kind);
          var error = translatable(msgId, msgData);
          return (0, _validators.errorReport)(kind, error);
        } else if ((0, _utils.objectIsEmpty)(trimmed)) return msg;else if (typeof trimmed == 'string') return translatable(trimmed);else return trimmed;
      } else {
        return msg;
      }
    }
  }, null);
};

/**
 * Validates a document using some validators and
 * a strategy.
 *
 * @param {object} doc Document to validate
 * @param {object} validationsByKey Object with the
 *    validators stored by field
 * @param {string} validationType 'error' or 'warning'
 * @param {object} props Pass props to your validators.
 *    Validator will receive it as its third parameter.
 *
 * @returns {object} Validation messages for ReduxForm.
 */
var validate = exports.validate = function validate(_ref2) {
  var model = _ref2.model,
    validationsByKey = _ref2.validationsByKey,
    validationType = _ref2.validationType,
    props = _ref2.props;
  var keys = Object.keys(validationsByKey);
  return keys.reduce(function (messages, key) {
    var validations = validationsByKey[key];
    var msg = validateEntry({
      validations: validations,
      value: model[key],
      validationType: validationType,
      props: props
    });
    if (msg && !(0, _utils.objectIsEmpty)(msg)) messages[key] = msg;
    return messages;
  }, {});
};

/**
 * Function that validates a document and produces
 * a translated or untranslated tree depending on
 * how it was created.
 *
 * @callback validator
 * @param {object} values Document
 *
 * @returns Tree with (translated or not) error or
 *    warning messages.
 */