"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.testFactory = exports.specialTypes = exports.errorReport = exports.addValidator = exports.addSpecialTypeValidator = void 0;
var _schema = require("./schema");
var _utils = require("./utils");
function _typeof(o) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) { return typeof o; } : function (o) { return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o; }, _typeof(o); }
/**
 * Validation creators for some types.
 */
var specialTypes = exports.specialTypes = {
  integer: function integer(value) {
    return value && !Number.isInteger(value);
  },
  select: function select(value) {
    return value && typeof value != 'string';
  },
  radios: function radios(value) {
    return value && typeof value != 'string';
  },
  password: function password(value) {
    return value && typeof value != 'string';
  }
};

/**
 * Adds a special type validation
 *
 * @param {string} type Type validated
 * @param {function} validator value => true_to_fail
 *    validator
 */
var addSpecialTypeValidator = exports.addSpecialTypeValidator = function addSpecialTypeValidator(type, validator) {
  specialTypes[type] = validator;
};

/**
 * Translates ArrayField error for ReduxForm
 *
 * @param {string} kind Error type
 * @param {string} error Error message
 *
 * @returns {object|string} Error message
 */
var errorReport = exports.errorReport = function errorReport(kind, error) {
  switch (kind) {
    case 'minChildren':
    case 'maxChildren':
      return {
        _error: error
      };
    default:
      return error;
  }
};

/**
 * Error creators with the validator factories.
 */
var testFactory = exports.testFactory = {
  type: function type(entry) {
    var type = entry.type;
    if (type instanceof _schema.Schema) {
      return function (value, validationType) {
        if (value) {
          if (_typeof(value) == 'object') return type.validateType(value, validationType);else return validationType == 'error';
        } else return false;
      };
    } else if (Array.isArray(type)) {
      var schema = type[0];
      return function (value, validationType) {
        if (value && Array.isArray(value)) {
          return value.map(function (child) {
            return schema.validateType(child, validationType);
          });
        } else return false;
      };
    } else {
      var typeStr = (0, _utils.schemaType)(type);
      var tester = specialTypes[typeStr];
      return function (value, validationType) {
        // We don't want to complain in the warn pass
        if (value && validationType == 'error') {
          if (tester) return tester(value);else return (0, _utils.getType)(value) != typeStr;
        } else return false;
      };
    }
  },
  min: function min(entry) {
    return function (value) {
      return value && value.length < entry.min;
    };
  },
  max: function max(entry) {
    return function (value) {
      return value && value.length > entry.max;
    };
  },
  required: function required(entry) {
    return function (value) {
      return !Boolean(value);
    };
  },
  minChildren: function minChildren(entry) {
    return function (value) {
      return !Array.isArray(value) || value.length < (entry.minChildren || 0);
    };
  },
  maxChildren: function maxChildren(entry) {
    return function (value) {
      return !Array.isArray(value) || entry.maxChildren && value.length > entry.maxChildren;
    };
  },
  validation: function validation(entry) {
    return function (value) {
      return entry.validation(value, entry);
    };
  },
  accept: function accept(entry) {
    return function (value) {
      return value ? !entry.accept.test(value) : false;
    };
  },
  reject: function reject(entry) {
    return function (value) {
      return value && typeof value == 'string' ? entry.reject.test(value) : false;
    };
  },
  options: function options(entry) {
    return function (value, _, props) {
      var options = typeof entry.options == 'function' ? entry.options(props) : entry.options;
      return value && options.reduce(function (invalid, op) {
        return invalid && (op.value || op) != value;
      }, true);
    };
  }
};

/**
 * Validator. Accepts a value and returns validation result.
 *
 * @typedef {function} validator
 * @param {any} value Value to be validated
 *
 * @returns {boolean|translatable|string} ``true`` if validation fails.
 *    If it's a string, the value will be printed directly,
 *    untranslated to the input. If the value is an object,
 *    it's considered a translatable.
 */

/**
 * Creates a specific validator.
 *
 * @typedef {function} validatorConstructor
 * @param {entry} A Schema specification for a field.
 *
 * @returns {validator}
 */

/**
 * Adds a test creator to the testFactory table, that way you can
 * use your own custom validations.
 *
 * @param {string} validationName The validation name for the
 *    test being created.
 * @param {validatorConstructor} validation Function that takes
 *    the entry and creates a validator to be used later.
 */
var addValidator = exports.addValidator = function addValidator(validationName, validation) {
  testFactory[validationName] = validation;
};